// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file BiCGSolver.h */

#ifndef AMDIS_BICGSOLVER_H
#define AMDIS_BICGSOLVER_H

#include "OEMSolver.h"
#include "MemoryManager.h"
#include "CreatorInterface.h"

namespace AMDiS {

  // ============================================================================
  // ===== class BiCGSolver =====================================================
  // ============================================================================

  /**
   * \ingroup Solver
   *
   * \brief
   * Solves a linear system by a stabilized BiCG method and can be used for 
   * symmetric system matrices.
   */
  template<typename VectorType>
  class BiCGSolver : public OEMSolver<VectorType>
  {
  public:
    MEMORY_MANAGED(BiCGSolver<VectorType>);

    /** \brief
     * Creator class used in the OEMSolverMap.
     */
    class Creator : public OEMSolverCreator<VectorType>
    {
    public:
      MEMORY_MANAGED(Creator);

      virtual ~Creator() {};

      /** \brief
       * Returns a new BiCGSolver object.
       */   
      OEMSolver<VectorType>* create() { 
	return NEW BiCGSolver<VectorType>(this->name); 
      };
    };

    /** \brief
     * constructor
     */
    BiCGSolver(std::string name);

    /** \brief
     * destructor
     */
    ~BiCGSolver();

  protected:
    /** \brief
     * realisation of OEMSolver::solveSystem
     */
    int solveSystem(MatVecMultiplier<VectorType> *mv, VectorType *x, VectorType *b);

    /** \brief
     * realisation of OEMSolver::init
     */
    void init();

    /** \brief
     * realisation of OEMSolver::exit
     */
    void exit();

  private:
    // pointer to memory needed for solveSystem
    VectorType *rstar, *d, *s, *CAd, *h;
  };

}

#include "BiCGSolver.hh"

#endif // AMDIS_BICGSOLVER_H
