// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file BiCGStab_M.h */

#ifndef BICGStab_M_H
#define BICGStab_M_H

namespace AMDiS {

  // ============================================================================
  // ===== class BiCGStab_M =====================================================
  // ============================================================================

  /**
   * \ingroup Solver
   *
   * \brief
   * Solves a linear system by a stabilized BiCG method.
   * Can be used for general matrices.
   */
  template<typename VectorType>
  class BiCGStab_M : public OEMSolver<VectorType>
  {
  public:
    MEMORY_MANAGED(BiCGStab_M<VectorType>);

    /** \brief
     * Creator class used in the OEMSolverMap.
     */
    class Creator : public OEMSolverCreator<VectorType>
    {
    public:
      MEMORY_MANAGED(Creator);

      virtual ~Creator() {};

      /** \brief
       * Returns a new BiCGStab_M object.
       */
      OEMSolver<VectorType>* create()
      {
	return NEW BiCGStab_M<VectorType>(this->name);
      };
    };

    /** \brief
     * constructor
     */
    BiCGStab_M(std::string name);

    /** \brief
     * destructor
     */
    ~BiCGStab_M();

  protected:
    /** \brief
     * Implements OEMSolver<VectorType>::init().
     */
    void init();

    /** \brief
     * Implements OEMSolver<VectorType>::exit().
     */
    void exit();

    /** \brief
     * Implements OEMSolver<VectorType>::solve().
     */
    int solveSystem(MatVecMultiplier<VectorType> *mv, VectorType *x, VectorType *b);

  private:
    // Pointers to memory needed for solveSystem
    VectorType *r, *rt, *p, *v, *t, *ph, *sh, *xmin;
  };

}

#include "BiCGStab_M.hh"

#endif // BICGStab_M_H
