// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file ElInfo2d.h */

#ifndef AMDIS_ELINFO2D_H
#define AMDIS_ELINFO2D_H

#include "ElInfo.h"
#include "MemoryManager.h"

namespace AMDiS {

  // ============================================================================
  // ===== class ElInfo2d =======================================================
  // ============================================================================

  /** \ingroup Traverse
   * \brief
   * ElInfo class for 2-dimensional elements (\ref Triangle).
   */
  class ElInfo2d : public ElInfo
  {
  public:
    MEMORY_MANAGED(ElInfo2d);

    /** \brief
     * Constructor. Calls ElInfo's protected Constructor.
     */
    ElInfo2d(Mesh* aMesh);

    ~ElInfo2d();

    /** \brief
     * 2-dimensional realisation of ElInfo's fillElInfo method.
     */
    void fillElInfo(int ichild, const ElInfo *elinfo_old);

    /** \brief
     * 2-dimensional realisation of ElInfo's fillMacroInfo method.
     */
    void fillMacroInfo(const MacroElement *);

    /** \brief
     * 2-dimensional realisation of ElInfo's worldToCoord method.
     */
    const int worldToCoord(const WorldVector<double>& w, DimVec<double>* l) const;

    /** \brief
     * 2-dimensional realisation of ElInfo's calcGrdLambda method.
     */
    double calcGrdLambda(DimVec<WorldVector<double> >& grd_lam);

    /** \brief
     * 2-dimensional realisation of ElInfo's getNormal method.
     */
    double getNormal(int side, WorldVector<double> &normal);

    /** \brief
     * 2-dimensional realisation of ElInfo's getElementNormal method.
     */
    double getElementNormal( WorldVector<double> &normal) const;

    void getRefSimplexCoords(VectorOfFixVecs<DimVec<double> > *coords) const
    {
      FUNCNAME("ElInfo2d::getRefSimplexCoords()");

      ERROR_EXIT("NOT YET\n");
    }

    void getSubElementCoords(VectorOfFixVecs<DimVec<double> > *coords,
			     int iChild) const 
    {
      FUNCNAME("ElInfo2d::getSubElementCoords()");

      ERROR_EXIT("NOT YET\n");
    }

  protected:
    /** \brief
     * Temp vectors for function \ref calcGrdLambda.
     */
    WorldVector<double> *e1, *e2, *normal;
  };

}

#endif // AMDIS_ELINFO2D_H
