#include "AMDiS.h"

using namespace std;
using namespace AMDiS;

// ===== function definitions // 
/** \brief
 * Dirichlet boundary function
 */
class G : public AbstractFunction<double, WorldVector<double> >
{
public:
  MEMORY_MANAGED(G);

  /** \brief
   * Implementation of AbstractFunction::operator().
   */
  const double& operator()(const WorldVector<double>& x) const
  {
    static double result = 0.0;
    return result;
  };
};

/** \brief
 * RHS function
 */
class F : public AbstractFunction<double, WorldVector<double> >
{
public:
  MEMORY_MANAGED(F);

  F(int degree) : AbstractFunction<double, WorldVector<double> >(degree) {};

  /** \brief
   * Implementation of AbstractFunction::operator().
   */
  const double& operator()(const WorldVector<double>& x) const {
    static double result = 0.0;
    int dim = x.getSize();
    double r2 = (x*x);
    double ux = exp(-10.0*r2);
    result = -(400.0*r2 - 20.0*dim)*ux;
    return result;
  };
};

// // ===== main program // 
int main(int argc, char* argv[])
{
  FUNCNAME("main");

  // ===== check for init file =====
  TEST_EXIT(argc == 2)("usage: periodic initfile\n");

  // ===== init parameters =====
  Parameters::init(false, argv[1]);

  // ===== create and init the scalar problem ===== 
  ProblemScal periodic("periodic");
  periodic.initialize(INIT_ALL);

  // === create adapt info ===
  AdaptInfo *adaptInfo = NEW AdaptInfo("periodic->adapt", 1);

  // === create adapt ===
  AdaptStationary *adapt = NEW AdaptStationary("periodic->adapt",
					       &periodic,
					       adaptInfo);
  
  // ===== add boundary conditions =====
  periodic.addDirichletBC(1, NEW G);

  // ===== add periodic conditions =====
  periodic.addPeriodicBC(-1);

  // ===== create matrix operator =====
  Operator matrixOperator(Operator::MATRIX_OPERATOR, periodic.getFESpace());
  matrixOperator.addSecondOrderTerm(NEW Laplace_SOT);
  periodic.addMatrixOperator(&matrixOperator);

  // ===== create rhs operator =====
  int degree = periodic.getFESpace()->getBasisFcts()->getDegree();
  Operator rhsOperator(Operator::VECTOR_OPERATOR, periodic.getFESpace());
  rhsOperator.addZeroOrderTerm(NEW CoordsAtQP_ZOT(NEW F(degree)));
  periodic.addVectorOperator(&rhsOperator);

  // ===== start adaption loop =====
  adapt->adapt();

  periodic.writeFiles(adaptInfo, true);
}


